Index: FL/Fl_Menu_.H
===================================================================
--- FL/Fl_Menu_.H	(revision 10483)
+++ FL/Fl_Menu_.H	(working copy)
@@ -164,6 +164,7 @@
   Fl_Color down_color() const {return selection_color();}
   /** For back compatibility, same as selection_color() */
   void down_color(unsigned c) {selection_color(c);}
+  void setonly(Fl_Menu_Item* item);
 };
 
 #endif
Index: src/Fl_Menu_.cxx
===================================================================
--- src/Fl_Menu_.cxx	(revision 10483)
+++ src/Fl_Menu_.cxx	(working copy)
@@ -254,8 +254,9 @@
   if (v) {
     if (v->radio()) {
       if (!v->value()) { // they are turning on a radio item
-	set_changed();
-	((Fl_Menu_Item*)v)->setonly();
+        set_changed();
+        //((Fl_Menu_Item*)v)->setonly();
+        setonly((Fl_Menu_Item*)v);
       }
       redraw();
     } else if (v->flags & FL_MENU_TOGGLE) {
@@ -276,7 +277,58 @@
   return v;
 }
 
-/** Turns the radio item "on" for the menu item and turns off adjacent radio items set. */
+/* returns pointer to first Fl_Menu_Item of the submenu containing item,
+ within a menu that starts at start. FL_SUBMENU_POINTER is properly handled.
+ If pnext is non-nil, *pnext is set upon return to item after submenu end.
+ */
+static Fl_Menu_Item *first_submenu_item(Fl_Menu_Item *item, Fl_Menu_Item *start, Fl_Menu_Item **pnext = NULL)
+{
+  Fl_Menu_Item *j = start;
+  while (j->label()) {
+    if (j == item) return start;
+    if (j->submenu()) {
+      Fl_Menu_Item *first;
+      if (j->flags&FL_SUBMENU_POINTER) {
+        first = (Fl_Menu_Item*)j->user_data();
+      } else {
+        first = j + 1;
+      }
+      first = first_submenu_item(item, first, &j);
+      if (first) return first;
+    }
+    else j++;
+  }
+  if (pnext) *pnext = j+1;
+  return NULL;
+}
+
+/** Turns the radio item "on" for the menu item and turns off adjacent radio items of the same group. */
+void Fl_Menu_::setonly(Fl_Menu_Item* item) {
+  item->flags |= FL_MENU_RADIO | FL_MENU_VALUE;
+  Fl_Menu_Item* j;
+  for (j = item; ; ) {	// go down
+    if (j->flags & FL_MENU_DIVIDER) break; // stop on divider lines
+    j++;
+    if (!j->text || !j->radio()) break; // stop after group
+    j->clear();
+  }
+  Fl_Menu_Item* first; // for the first item of the menu containing item
+  if (item >= menu_ && item < menu_ + size()) {
+    first = menu_;
+  } else { // FL_SUBMENU_POINTER is involved
+    first = first_submenu_item(item, menu_);
+  }
+  for (j = item-1; j>=first; j--) { // go up
+    //DEBUG printf("GO UP: WORKING ON: item='%s', flags=%x\n", j->text, j->flags);
+    if (!j->text || (j->flags&FL_MENU_DIVIDER) || !j->radio()) break;
+    j->clear();
+  }
+}
+
+/** Turns the radio item "on" for the menu item and turns off adjacent radio items set.
+ Deprecated: this method is dangerous if radio items are first in the menu.
+ Use Fl_Menu_::setonly(Fl_Menu_Item*) instead.
+ */
 void Fl_Menu_Item::setonly() {
   flags |= FL_MENU_RADIO | FL_MENU_VALUE;
   Fl_Menu_Item* j;
Index: test/menubar.cxx
===================================================================
--- test/menubar.cxx	(revision 10483)
+++ test/menubar.cxx	(working copy)
@@ -207,6 +207,7 @@
     char buf[100];
     sprintf(buf,"item %d",i);
     hugemenu[i].text = strdup(buf);
+    hugemenu[i].flags = FL_MENU_RADIO;
   }
   Fl_Double_Window window(WIDTH,400);
   window.callback(window_cb);
